#!/bin/bash
#
# Small script to archive a Maildir as a Courier subfolder tree.
# Archive subfolder name created from Maildir's parent folder's name.
# Part of vexim-addons package.
#
# Call with domain/mailbox relative pathname parameter as root.
# Example: maildir-archive.sh "my.domain/my.user"
# on call: my.user/Maildir/cur /new /tmp (etc.)
#                         /.Drafts/cur /new /tmp (etc.)
#			  [...]
# goes to: my.user/Maildir/.my_user/.Inbox/cur /new /tmp (etc.)
#                         /.my_user/.Drafts/cur /new /tmp (etc.)
#                         [...]
# and empty mail folder:  /.my_user/cur /new /tmp (etc.) created also.
# Doesn't overwrite existing folders. Does nothing on already archived Maildir.

DATE=$(which date)			# date command call
EXIT_ERR=1				# Exit code on failure
GREP=$(which grep)			# grep command call
MAILDIRMAKE=$(which maildirmake)	# maildirmake command call
MAILROOT="/var/mail/.vexim"		# Mail storage root path
SED=$(which sed)			# sed command call

MSG_VEXIM_NOMAILDIR="Maildir isn't found at"
MSG_USAGE="Usage: $0 relative_pathname_to_user's_folder as root."

# Mandatory parameter: relative pathname.
PATHNAME=$1; shift
if [ -z "$PATHNAME" ]; then echo -e "$MSG_USAGE"; exit $EXIT_ERR; fi
# Turn to absolute pathname.
PATHNAME="$MAILROOT/$PATHNAME"
# Check corresponding Maildir.
MAILDIR="$PATHNAME/Maildir"
if [ ! -d "$MAILDIR" ]; then
    echo -e "$MSG_VEXIM_NOMAILDIR $MAILDIR" >&2
    exit $EXIT_ERR
fi

# Courier subfolder name is mailbox folder's name,
# but any dots changed to underscores.
ARCHNAME=".$(echo $(basename $PATHNAME) | $SED "s/\./_/g")"

# Creating new Inbox within Maildir (with unique name if exists)
# preserving access rights and moving contents into.
# Does nothing on already archived structure.
INBOX="$MAILDIR/.Inbox"
if [ -e "$INBOX" ]; then INBOX="$INBOX-$($DATE +%s)"; fi
for content in $(ls -1 "$MAILDIR") # non-dot files only
do
    if [ ! -d "$INBOX" ]; then
	mkdir "$INBOX"; chown --reference "$MAILDIR" "$INBOX"; chmod --reference "$MAILDIR" "$INBOX"
    fi
    mv "$MAILDIR/$content" "$INBOX/"
done

# Renaming all Courier folders, insering ARCHNAME.
# Creating an empty ARCHNAME Courier folder if necessary.
# Does nothing on already archived structure.
for content in $(ls -1A "$MAILDIR")
do
    if [ -z "$(echo "$content" | $GREP "^$ARCHNAME")" ]; then
	# Creaing parent folder if necessary.
	if [ ! -d "$MAILDIR/$ARCHNAME" ]; then
	    $MAILDIRMAKE -f "${ARCHNAME:1}" "$MAILDIR"
	    chown --reference "$MAILDIR" "$MAILDIR/$ARCHNAME"
	    chmod --reference "$MAILDIR" "$MAILDIR/$ARCHNAME"
	fi
	# Renaming content avoiding overwrite anything.
	if [ ! -e "$MAILDIR/$ARCHNAME$content" ]; then
	    mv "$MAILDIR/$content" "$MAILDIR/$ARCHNAME$content"
	else
	    mv "$MAILDIR/$content" "$MAILDIR/$ARCHNAME$content-$($DATE +%s)"
	fi
    fi
done

# That's all, folks!
